==> tag_1.0.0.zip: server/src/main/java/cn/com/qmth/scancloud/controller/admin/ExamController.java <==
package cn.com.qmth.scancloud.controller.admin;

import java.util.HashMap;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ReentrantException;
import com.qmth.boot.mybatis.query.BaseQuery;

import cn.com.qmth.scancloud.bean.ExamConfigDomain;
import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.controller.BaseController;
import cn.com.qmth.scancloud.entity.ExamEntity;
import cn.com.qmth.scancloud.enums.LockType;
import cn.com.qmth.scancloud.exception.ParameterExceptions;
import cn.com.qmth.scancloud.service.ExamService;
import cn.com.qmth.scancloud.service.StudentService;
import cn.com.qmth.scancloud.task.ExamStudentSyncThread;
import cn.com.qmth.scancloud.vo.ExamConfigVo;
import cn.com.qmth.scancloud.vo.ExamRemoteVo;
import cn.com.qmth.scancloud.vo.ExamUploadVo;
import cn.com.qmth.scancloud.vo.ExamVo;
import cn.com.qmth.scancloud.vo.examinfo.ExamInfoVo;
import cn.com.qmth.scancloud.vo.examinfo.ExamQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.sf.json.JSONObject;

@RestController
@Api(tags = "考试接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/exam")
@Aac(strict = BOOL.FALSE, auth = BOOL.TRUE)
public class ExamController extends BaseController {

    @Autowired
    private ExamService examService;
    
    @Autowired
    private StudentService studentService;

    @Resource
    private ConcurrentService concurrentService;

    @Autowired
    private AsyncTaskExecutor taskExecutor;

    @ApiOperation(value = "获取考试详情")
    @RequestMapping(value = "/info", method = RequestMethod.POST)
    public ExamInfoVo info(@RequestParam Long id) {
    	User user = getAccessUser();
        return examService.getExamInfoVo(id,user);
    }

    @ApiOperation(value = "获取考试列表")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public PageResult<ExamVo> list(ExamQuery query) {
    	User user = getAccessUser();
        return examService.pageQuery(query,user);
    }

    @ApiOperation(value = "启用/关闭考试")
    @RequestMapping(value = "/toggle", method = RequestMethod.POST)
    public JSONObject enable(@RequestParam Long id, @RequestParam Boolean enable) {
    	User user = getAccessUser();
        JSONObject result = new JSONObject();
        ExamEntity exam = examService.getById(id);
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (!exam.getSchoolId().equals(user.getSchoolId())) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        exam.setEnable(enable);
        examService.saveOrUpdate(exam);
        result.accumulate("enable", enable);
        result.accumulate("updateTime", exam.getUpdateTime());
        return result;
    }

    @ApiOperation(value = "未启用考试")
    @RequestMapping(value = "/remote/list", method = RequestMethod.POST)
    public PageResult<ExamRemoteVo> remote(BaseQuery<?> query) {
        User user = getAccessUser();
        return examService.listRemote(query, user);
    }

    @ApiOperation(value = "启用新考试")
    @RequestMapping(value = "/init", method = RequestMethod.POST)
    public Map<String, Object> init(ExamConfigDomain domain) {
        User user = getAccessUser();
        ExamConfigVo vo=examService.init(user, domain);
        Map<String, Object> result = new HashMap<String, Object>();
        result.put("updateTime", vo.getUpdateTime());
        return result;
    }

    @ApiOperation(value = "修改考试参数")
    @RequestMapping(value = "/config", method = RequestMethod.POST)
    public ExamConfigVo config(ExamConfigDomain domain) {
        User user = getAccessUser();
        return examService.config(user, domain);
    }

    @ApiOperation(value = "修改考试上传状态")
    @RequestMapping(value = "/upload", method = RequestMethod.POST)
    public ExamUploadVo upload(@RequestParam Long id, @RequestParam Boolean enable) {
    	User user = getAccessUser();
        ExamUploadVo vo = new ExamUploadVo();
        ExamEntity exam = examService.getById(id);
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (!exam.getSchoolId().equals(user.getSchoolId())) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        exam.setEnableUpload(enable);
        examService.saveOrUpdate(exam);
        vo.setEnable(exam.getEnableUpload());
        vo.setUpdateTime(exam.getUpdateTime());
        vo.setProgress(studentService.getUploadProgress(id));
        return vo;
    }

    @ApiOperation(value = "同步考务数据")
    @RequestMapping(value = "/data/sync", method = RequestMethod.POST)
    public JSONObject sync(@RequestParam Long examId) {
        User user = getAccessUser();
        ExamEntity exam = examService.getById(examId);
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (!exam.getSchoolId().equals(user.getSchoolId())) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (!concurrentService.isLocked(LockType.EXAM_SYNC + "-" + examId)) {
            taskExecutor.submit(new ExamStudentSyncThread(getAccessUser(), examId, examService, concurrentService));
        } else {
            throw new ReentrantException("正在同步，请稍后再试");
        }
        JSONObject result = new JSONObject();
        result.put("synching", concurrentService.isLocked(LockType.EXAM_SYNC + "-" + examId));
        result.put("progress", examService.syncProgress(user, examId));
        result.put("updateTime", examService.getById(examId).getUpdateTime());
        return result;
    }

    @ApiOperation(value = "考务数据同步状态")
    @RequestMapping(value = "/data/sync/status", method = RequestMethod.POST)
    public Map<String, Object> syncStatus(@RequestParam Long examId) {
        User user = getAccessUser();
        ExamEntity exam = examService.getById(examId);
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (!exam.getSchoolId().equals(user.getSchoolId())) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        Map<String, Object> result = new HashMap<String, Object>();
        result.put("synching", concurrentService.isLocked(LockType.EXAM_SYNC + "-" + examId));
        result.put("progress", examService.syncProgress(user, examId));
        result.put("updateTime", examService.getById(examId).getUpdateTime());
        return result;
    }
}

==> tag_1.0.2.zip: server/src/main/java/cn/com/qmth/scancloud/controller/admin/ExamController.java <==
package cn.com.qmth.scancloud.controller.admin;

import cn.com.qmth.scancloud.bean.ExamConfigDomain;
import cn.com.qmth.scancloud.bean.SubjectConfigDomain;
import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.controller.BaseController;
import cn.com.qmth.scancloud.entity.ExamEntity;
import cn.com.qmth.scancloud.enums.LockType;
import cn.com.qmth.scancloud.exception.ParameterExceptions;
import cn.com.qmth.scancloud.service.ExamService;
import cn.com.qmth.scancloud.service.StudentService;
import cn.com.qmth.scancloud.service.SubjectService;
import cn.com.qmth.scancloud.task.ExamDataSyncThread;
import cn.com.qmth.scancloud.vo.*;
import cn.com.qmth.scancloud.vo.examinfo.ExamInfoVo;
import cn.com.qmth.scancloud.vo.examinfo.ExamQuery;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ReentrantException;
import com.qmth.boot.mybatis.query.BaseQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.sf.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

@RestController
@Api(tags = "考试接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/exam")
@Aac(strict = BOOL.FALSE, auth = BOOL.TRUE)
public class ExamController extends BaseController {

    @Autowired
    private ExamService examService;

    @Autowired
    private StudentService studentService;

    @Resource
    private ConcurrentService concurrentService;

    @Autowired
    private AsyncTaskExecutor taskExecutor;

    @Autowired
    private SubjectService subjectService;

    @ApiOperation(value = "获取考试详情")
    @RequestMapping(value = "/info", method = RequestMethod.POST)
    public ExamInfoVo info(@RequestParam Long id) {
        User user = getAccessUser();
        return examService.getExamInfoVo(id, user);
    }

    @ApiOperation(value = "获取考试列表")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public PageResult<ExamVo> list(ExamQuery query) {
        User user = getAccessUser();
        return examService.pageQuery(query, user);
    }

    @ApiOperation(value = "启用/关闭考试")
    @RequestMapping(value = "/toggle", method = RequestMethod.POST)
    public JSONObject enable(@RequestParam Long id, @RequestParam Boolean enable) {
        User user = getAccessUser();
        JSONObject result = new JSONObject();
        ExamEntity exam = examService.getById(id);
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (!exam.getSchoolId().equals(user.getSchoolId())) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        exam.setEnable(enable);
        exam.setUpdaterId(user.getId());
        exam.setUpdateTime(System.currentTimeMillis());
        examService.saveOrUpdate(exam);
        result.accumulate("enable", enable);
        result.accumulate("updateTime", exam.getUpdateTime());
        return result;
    }

    @ApiOperation(value = "未启用考试")
    @RequestMapping(value = "/remote/list", method = RequestMethod.POST)
    public PageResult<ExamRemoteVo> remote(BaseQuery<?> query) {
        User user = getAccessUser();
        return examService.listRemote(query, user);
    }

    @ApiOperation(value = "启用新考试")
    @RequestMapping(value = "/init", method = RequestMethod.POST)
    public ExamInitVo init(@Validated(ExamConfigDomain.ExamConfigInit.class) ExamConfigDomain domain) {
        User user = getAccessUser();
        ExamInitVo vo = examService.init(user, domain);
        return vo;
    }

    @ApiOperation(value = "修改考试参数")
    @RequestMapping(value = "/config", method = RequestMethod.POST)
    public ExamConfigVo config(@Validated ExamConfigDomain domain) {
        User user = getAccessUser();
        return examService.config(user, domain);
    }

    @ApiOperation(value = "修改考试上传状态")
    @RequestMapping(value = "/upload", method = RequestMethod.POST)
    public ExamUploadVo upload(@RequestParam Long id, @RequestParam Boolean enable) {
        User user = getAccessUser();
        ExamUploadVo vo = new ExamUploadVo();
        ExamEntity exam = examService.getById(id);
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (!exam.getSchoolId().equals(user.getSchoolId())) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        exam.setEnableUpload(enable);
        examService.saveOrUpdate(exam);
        vo.setEnable(exam.getEnableUpload());
        vo.setUpdateTime(exam.getUpdateTime());
        vo.setProgress(studentService.getUploadProgress(id));
        return vo;
    }

    @ApiOperation(value = "同步考务数据")
    @RequestMapping(value = "/data/sync", method = RequestMethod.POST)
    public JSONObject sync(@RequestParam Long examId) {
        User user = getAccessUser();
        ExamEntity exam = examService.getById(examId);
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (!exam.getSchoolId().equals(user.getSchoolId())) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (!concurrentService.isLocked(LockType.EXAM_SYNC + "-" + examId)) {
            taskExecutor.submit(new ExamDataSyncThread(getAccessUser(), examId, examService, concurrentService));
        } else {
            throw new ReentrantException("正在同步，请稍后再试");
        }
        JSONObject result = new JSONObject();
        result.put("synching", concurrentService.isLocked(LockType.EXAM_SYNC + "-" + examId));
        result.put("progress", examService.syncProgress(user, examId));
        result.put("updateTime", examService.getById(examId).getDataSyncTime());
        return result;
    }

    @ApiOperation(value = "考务数据同步状态")
    @RequestMapping(value = "/data/sync/status", method = RequestMethod.POST)
    public Map<String, Object> syncStatus(@RequestParam Long examId) {
        User user = getAccessUser();
        ExamEntity exam = examService.getById(examId);
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (!exam.getSchoolId().equals(user.getSchoolId())) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        Map<String, Object> result = new HashMap<String, Object>();
        result.put("synching", concurrentService.isLocked(LockType.EXAM_SYNC + "-" + examId));
        result.put("progress", examService.syncProgress(user, examId));
        result.put("updateTime", examService.getById(examId).getDataSyncTime());
        return result;
    }

    @ApiOperation(value = "修改考试科目参数")
    @RequestMapping(value = "/subject/config", method = RequestMethod.POST)
    public SubjectConfigVo subjectConfig(@Validated SubjectConfigDomain domain) {
        User user = getAccessUser();
        return subjectService.config(user, domain);
    }
}
